package live.aaqua.awscrtexample.config;

import java.net.URI;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import software.amazon.awssdk.auth.credentials.AwsBasicCredentials;
import software.amazon.awssdk.auth.credentials.StaticCredentialsProvider;
import software.amazon.awssdk.http.async.SdkAsyncHttpClient;
import software.amazon.awssdk.http.crt.AwsCrtAsyncHttpClient;
import software.amazon.awssdk.regions.Region;
import software.amazon.awssdk.services.dynamodb.DynamoDbAsyncClient;
import software.amazon.awssdk.services.dynamodb.model.DescribeTableRequest;

@Configuration
public class DynamoDbConfig {

  private static final Logger LOG = LogManager.getLogger(DynamoDbConfig.class);

  @Value("${dynamodb.region}")
  private String region;

  @Value("${dynamodb.endpoint:#{null}}")
  private String endpoint;

  @Value("${dynamodb.access-key-id:#{null}}")
  private String accessKeyId;

  @Value("${dynamodb.secret-access-key:#{null}}")
  private String secretAccessKey;


  @Value("${dynamodb.table}")
  private String table;

  @Bean
  public DynamoDbAsyncClient dynamoDbAsyncClient() {
    LOG.info("Start loading DynamoDB Client");

    LOG.debug("Create AwsCrtAsyncHttpClient");
    SdkAsyncHttpClient httpClient = AwsCrtAsyncHttpClient.create();

    LOG.debug("Create DynamoDbAsyncClient with an AwsCrtAsyncHttpClient");
    var client = DynamoDbAsyncClient.builder()
        .region(Region.of(region))
        .httpClient(httpClient)
        .endpointOverride(URI.create(endpoint))
        .credentialsProvider(StaticCredentialsProvider.create(AwsBasicCredentials.create(accessKeyId, secretAccessKey))).build();

    try {
      LOG.info("Try to describe the table to check connection with DynamoDB Table");
      client.describeTable(DescribeTableRequest.builder()
              .tableName(table)
              .build())
          .join();
    } catch (Exception e) {
      LOG.error("Was not able to load describe table. {}", e.getMessage(), e);
    }

    LOG.info("Successful created dynamoDB Client");
    return client;
  }

}
